macro_rules! byte_slice_as_type {
($slice:ident, $type:ty) => { ... };
}Expand description
A macro for creating a view of an immutable slice of bytes as a different type.
macro_rules! byte_slice_as_type {
($slice:ident, $type:ty) => { ... };
}A macro for creating a view of an immutable slice of bytes as a different type.