Trait AdapterIterators

pub trait AdapterIterators<'a, T: 'a> {
    // Required methods
    fn iter_channel(&self, channel: usize) -> Option<ChannelSamples<'a, '_, T>>;
    fn iter_channels(&self) -> Channels<'a, '_, T>;
    fn iter_frame(&self, frame: usize) -> Option<FrameSamples<'a, '_, T>>;
    fn iter_frames(&self) -> Frames<'a, '_, T>;
}
A trait providing convenient iteration through frames and/or channels of an Adapter.

Required Methods

fn iter_channel(&self, channel: usize) -> Option<ChannelSamples<'a, '_, T>>

Get an iterator that yields the sample value of the specified channel.

fn iter_channels(&self) -> Channels<'a, '_, T>

Get an iterator that yields iterators for the channels.

fn iter_frame(&self, frame: usize) -> Option<FrameSamples<'a, '_, T>>

Get an iterator that yields the sample values of the specified frame.

fn iter_frames(&self) -> Frames<'a, '_, T>

Get an iterator that yields iterators for the frames.

Implementors

Source

impl<'a, T, U> AdapterIterators<'a, T> for U
where T: Clone + 'a, U: Adapter<'a, T>,